﻿using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using Castle.Components.DictionaryAdapter;

namespace Escali.LicenseControl.Client.ClientModels
{
    public class CustomerClientModel
    {
        public int? CustomerSeq { get; set; }

        [Required(ErrorMessage = "Obligatorisk felt")]
        [MaxLength(11)]
        public string OrganizationNumber { get; set; }
        
        [Required(ErrorMessage = "Obligatorisk felt")]
        [MaxLength(50)]
        public string CustomerName { get; set; }
        
        public string? LedgerNumber { get; set; }
        
        [MaxLength(100)]
        public string? Connection { get; set; }
        
        [MaxLength(50)]
        public string? DBName { get; set; }
        
        [MaxLength(50)]
        public string? CustomerUsername { get; set; }
        
        [MaxLength(50)]
        public string? CustomerPassword { get; set; }
        
        [MaxLength(50)]
        public string? API { get; set; }

        [Required(ErrorMessage = "Obligatorisk felt")]
        public DateTime CustomerSince { get; set; }

        public bool? IsActive { get; set; }
        
        public int? PrioritySeq { get; set; }
        public PriorityClientModel? Priority { get; set; }
        
        public int? SegmentSeq { get; set; }
        public SegmentClientModel? Segment { get; set; }
        
        public int? RegionSeq { get; set; }
        [ValidateComplexType]
        public RegionClientModel Region { get; set; } = new RegionClientModel();
        
        public int? UserSeq { get; set; }
        public UserClientModel? User { get; set; }

        public List<AgreementClientModel>? Agreements { get; set; }

        public List<AgreementElementClientModel> AllAgreements => GetAllAgreements();

        private List<AgreementElementClientModel> GetAllAgreements()
        {
            List<AgreementElementClientModel> agreements = new List<AgreementElementClientModel>();
            Agreements?.ForEach(a => a.AgreementElements?.ForEach(ae => agreements.Add(ae)));
            return agreements;
        }
        
        
    }
}
