using Escali.LicenseControl.Client.State;
using Escali.LicenseControl.Client.ViewModels;
using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Utils;
using Microsoft.EntityFrameworkCore;

var builder = WebApplication.CreateBuilder(args);

// Add services to the container.
builder.Services.AddRazorPages();
builder.Services.AddServerSideBlazor();
builder.Services.AddHttpClient();

builder.Services.AddDbContext<DataContext>(options =>
{
    //options.UseSqlServer("name=ConnectionStrings:Default");
    options.UseInMemoryDatabase(databaseName: "Escali");
    options.UseQueryTrackingBehavior(QueryTrackingBehavior.NoTracking);
}, ServiceLifetime.Transient);



// AutoMapper
builder.Services.AddAutoMapper(AppDomain.CurrentDomain.GetAssemblies());

// Services
builder.Services.AddScoped<StateService>();
builder.Services.AddScoped<AlertService>();

builder.Services.AddScoped<CurrencyViewModel>();
builder.Services.AddScoped<ProductViewModel>();
builder.Services.AddScoped<ModuleViewModel>();
builder.Services.AddScoped<SegmentViewModel>();
builder.Services.AddScoped<MainSegmentViewModel>();
builder.Services.AddScoped<PriceListViewModel>();
builder.Services.AddScoped<UserViewModel>();
builder.Services.AddScoped<CustomerViewModel>();
builder.Services.AddScoped<PriorityViewModel>();
builder.Services.AddScoped<RegionViewModel>();
builder.Services.AddScoped<PriceElementViewModel>();
builder.Services.AddScoped<AgreementViewModel>();
builder.Services.AddScoped<AgreementElementViewModel>();
builder.Services.AddScoped<ModuleLevelViewModel>();

var app = builder.Build();

var scope = app.Services.CreateScope();
var service = scope.ServiceProvider;
var context = service.GetRequiredService<DataContext>();
DataGenerator.Initialize(service);


// Configure the HTTP request pipeline.
if (!app.Environment.IsDevelopment())
{
    app.UseExceptionHandler("/Error");
    // The default HSTS value is 30 days. You may want to change this for production scenarios, see https://aka.ms/aspnetcore-hsts.
    app.UseHsts();
}

app.Use(async (context, next) =>
{
    context.Response.Headers.Add("X-Frame-Options", "SAMEORIGIN");
    await next();
});

app.UseHttpsRedirection();

app.UseStaticFiles();

app.UseRouting();

app.MapBlazorHub();
app.MapFallbackToPage("/_Host");

app.Run();