using AutoMapper;
using Escali.LicenseControl.Client.ClientModels;

namespace Escali.LicenseControl.Client.State;

public class StateService
{
    private IMapper _mapper;
    public StateService(IMapper mapper)
    {
        _mapper = mapper;
    }
    public StateProperty<int> SelectedProductSeq { get; } = new();
    public StateProperty<int> SelectedPriceListSeq { get; } = new();
    public StateProperty<int> SelectedUserSeq { get; } = new();
    public StateProperty<string> HeaderName { get; } = new();

    public StateProperty<CustomerClientModel> EditCustomer { get; } = new();
    public StateProperty<PriceListClientModel> EditPriceList { get; } = new();
    public StateProperty<ModuleClientModel> EditModule { get; } = new();
    public StateProperty<AgreementElementClientModel> EditAgreementElement { get; } = new();
    public StateProperty<int?> AddAgreementCustomer { get; } = new();

    public void UpdateEditCustomer(CustomerClientModel customer)
    {
        EditCustomer.Value = _mapper.Map<CustomerClientModel>(customer);
    }
    
    public void UpdateEditPriceList(PriceListClientModel priceList)
    {
        EditPriceList.Value = _mapper.Map<PriceListClientModel>(priceList);
    }
    
    public void UpdateEditModule(ModuleClientModel module)
    {
        EditModule.Value = _mapper.Map<ModuleClientModel>(module);
        if (EditModule.Value != null) EditModule.Value.ModuleLevels = _mapper.Map<List<ModuleLevelClientModel>>(module.ModuleLevels);
    }

    public void UpdateAgreementElement(AgreementElementClientModel agreementElement)
    {
        EditAgreementElement.Value = _mapper.Map<AgreementElementClientModel>(agreementElement);
    }
    
    public void UpdateAddAgreementCustomer(int customerSeq)
    {
        AddAgreementCustomer.Value = customerSeq;
    }
    
}