using AutoMapper;
using Escali.LicenseControl.Client.State;
using Escali.LicenseControl.Client.Utils;
using Escali.LicenseControl.Client.ClientModels;
using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Models;
using Escali.LicenseControl.DataAccess.Services;
using Microsoft.EntityFrameworkCore;

namespace Escali.LicenseControl.Client.ViewModels
{
    /// <summary>
    /// AgreementElementViewService class has methods for creating and updating AgreementElements
    /// </summary>
    public class AgreementElementViewModel : IViewModel
    {

        private readonly AgreementElementService _agreementElementService;
        private AlertService _alertService;

        public List<AgreementElementClientModel>? AgreementElements { get; private set; }
        public AgreementElementViewModel(DataContext context, IMapper mapper, AlertService alertService)
        {
            _agreementElementService = new AgreementElementService(context);
            _mapper = mapper;
            _alertService = alertService;
        }

        public async Task GetAgreementElements()
        {
            var agreementElements = await _agreementElementService.GetAllAgreementElements();
            UpdateAgreementElements(_mapper.Map<List<AgreementElementClientModel>>(agreementElements));
        }

        public async Task<AgreementElementClientModel?> CreateAgreementElement(AgreementElementClientModel agreementElementView)
        {
            var mapped = _mapper.Map<AgreementElement>(agreementElementView);
            try
            {
                var res = await _agreementElementService.AddAgreementElement(mapped);
                _alertService.UpdateInfoMessage(new InfoMessage("Ny avtale opprettet", InfoMessageType.SUCCESS));

                var updatedAgreementElements = AgreementElements ?? new List<AgreementElementClientModel>();
                updatedAgreementElements.Add(_mapper.Map<AgreementElementClientModel>(res));

                UpdateAgreementElements(updatedAgreementElements);
                return _mapper.Map<AgreementElementClientModel>(res);
            }
            catch (DbUpdateException e)
            {
                _alertService.UpdateInfoMessage(new InfoMessage("Det oppstod en feil i database.", InfoMessageType.ERROR));
                return null;
            }
            catch (InvalidOperationException e)
            {
                _alertService.UpdateInfoMessage(new InfoMessage(e.Message, InfoMessageType.ERROR));
                return null;
            }
        }

        public async Task<AgreementElementClientModel>? UpdateAgreementElement(AgreementElementClientModel agreementElementView)
        {
            var mapped = _mapper.Map<AgreementElement>(agreementElementView);
            try
            {
                var res = await _agreementElementService.UpdateAgreementElement(mapped);
                // _alertService.UpdateInfoMessage(new InfoMessage("Modul ble oppdatert!", InfoMessageType.SUCCESS));

                var updatedAgreementElements = AgreementElements!.Select(m => m.AgreementElementSeq != agreementElementView.AgreementElementSeq ? m : _mapper.Map<AgreementElementClientModel>(res)).ToList();
                UpdateAgreementElements(updatedAgreementElements);
                return _mapper.Map<AgreementElementClientModel>(res);
            }
            catch (DbUpdateException e)
            {
                Logger.Error(e.InnerException.Message);
                _alertService.UpdateInfoMessage(new InfoMessage("Det oppstod en feil i database.", InfoMessageType.ERROR));
                return null;
            }
            catch (InvalidOperationException e)
            {
                Logger.Error(e.Message);
                _alertService.UpdateInfoMessage(new InfoMessage(e.Message, InfoMessageType.ERROR));
                return null;
            }
        }

        private void UpdateAgreementElements(List<AgreementElementClientModel> agreementElements)
        {
            AgreementElements = agreementElements;
            OnDataChanged();
        }

    }
}