using AutoMapper;
using Escali.LicenseControl.Client.State;
using Escali.LicenseControl.Client.Utils;
using Escali.LicenseControl.Client.ClientModels;
using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Models;
using Escali.LicenseControl.DataAccess.Services;
using Microsoft.EntityFrameworkCore;

namespace Escali.LicenseControl.Client.ViewModels
{
    public class AgreementViewModel : IViewModel
    {

        private readonly AgreementService _agreementService;
        private AlertService _alertService;

        public List<AgreementClientModel>? Agreements { get; private set; }
        public AgreementViewModel(DataContext context, IMapper mapper, AlertService alertService)
        {
            _agreementService = new AgreementService(context);
            _mapper = mapper;
            _alertService = alertService;
        }

        public async Task GetAgreements()
        {
            var agreements = await _agreementService.GetAllAgreements();
            UpdateAgreements(_mapper.Map<List<AgreementClientModel>>(agreements));
        }

        public async Task<AgreementClientModel?> CreateAgreement(AgreementClientModel agreementView)
        {
            var mapped = _mapper.Map<Agreement>(agreementView);
            try
            {
                var res = await _agreementService.AddAgreement(mapped);
                // _alertService.UpdateInfoMessage(new InfoMessage("Ny modul lagt til!", InfoMessageType.SUCCESS));

                var updatedAgreements = Agreements ?? new List<AgreementClientModel>();
                updatedAgreements.Add(_mapper.Map<AgreementClientModel>(res));

                UpdateAgreements(updatedAgreements);
                return _mapper.Map<AgreementClientModel>(res);
            }
            catch (DbUpdateException e)
            {
                _alertService.UpdateInfoMessage(new InfoMessage("Det oppstod en feil i database.", InfoMessageType.ERROR));
                return null;
            }
            catch (InvalidOperationException e)
            {
                _alertService.UpdateInfoMessage(new InfoMessage(e.Message, InfoMessageType.ERROR));
                return null;
            }
        }

        public async Task<AgreementClientModel>? UpdateAgreement(AgreementClientModel agreementView)
        {
            var mapped = _mapper.Map<Agreement>(agreementView);
            try
            {
                var res = await _agreementService.UpdateAgreement(mapped);
                // _alertService.UpdateInfoMessage(new InfoMessage("Modul ble oppdatert!", InfoMessageType.SUCCESS));

                var updatedAgreements = Agreements!.Select(m => m.AgreementSeq != agreementView.AgreementSeq ? m : _mapper.Map<AgreementClientModel>(res)).ToList();
                UpdateAgreements(updatedAgreements);
                return _mapper.Map<AgreementClientModel>(res);
            }
            catch (DbUpdateException e)
            {
                Logger.Error(e.InnerException.Message);
                _alertService.UpdateInfoMessage(new InfoMessage("Det oppstod en feil i database.", InfoMessageType.ERROR));
                return null;
            }
            catch (InvalidOperationException e)
            {
                Logger.Error(e.Message);
                _alertService.UpdateInfoMessage(new InfoMessage(e.Message, InfoMessageType.ERROR));
                return null;
            }
        }

        private void UpdateAgreements(List<AgreementClientModel> agreements)
        {
            Agreements = agreements;
            OnDataChanged();
        }

    }
}