﻿using AutoMapper;
using Escali.LicenseControl.Client.State;
using Escali.LicenseControl.Client.Utils;
using Escali.LicenseControl.Client.ClientModels;
using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Models;
using Escali.LicenseControl.DataAccess.Services;
using Microsoft.EntityFrameworkCore;

namespace Escali.LicenseControl.Client.ViewModels
{
    public class ModuleLevelViewModel : IViewModel
    {
        private readonly ModuleLevelService _modelLevelService;
        private AlertService _alertService;

        public List<ModuleLevelClientModel>? ModuleLevels { get; set; }

        public ModuleLevelViewModel(DataContext context, IMapper mapper, AlertService alertService)
        {
            _modelLevelService = new ModuleLevelService(context);
            _mapper = mapper;
            _alertService = alertService;
        }

        public async Task GetModuleLevels()
        {
            var moduleLevels = await _modelLevelService.GetAllModuleLevels();
            UpdateModuleLevels(_mapper.Map<List<ModuleLevelClientModel>>(moduleLevels));
        }

        public async Task<ModuleLevelClientModel?> AddModuleLevel(ModuleLevelClientModel moduleLevel)
        {
            var mapped = _mapper.Map<ModuleLevel>(moduleLevel);
            try
            {
                var res = await _modelLevelService.AddModuleLevel(mapped);
                _alertService.UpdateInfoMessage(new InfoMessage("Nytt modulnivå opprettet", InfoMessageType.SUCCESS));

                var updatedModuleLevels = ModuleLevels ?? new List<ModuleLevelClientModel>();
                updatedModuleLevels.Add(_mapper.Map<ModuleLevelClientModel>(res));

                UpdateModuleLevels(updatedModuleLevels);
                return _mapper.Map<ModuleLevelClientModel>(res);
            } 
            catch (DbUpdateException e) 
            {
                _alertService.UpdateInfoMessage(new InfoMessage("Det oppstod en feil i database.", InfoMessageType.ERROR));
                return null;
            }
        }

        private void UpdateModuleLevels(List<ModuleLevelClientModel> moduleLevels)
        {
            ModuleLevels = moduleLevels;
            OnDataChanged();
        }

    }
}
