using AutoMapper;
using Escali.LicenseControl.Client.State;
using Escali.LicenseControl.Client.Utils;
using Escali.LicenseControl.Client.ClientModels;
using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Models;
using Escali.LicenseControl.DataAccess.Services;
using Microsoft.EntityFrameworkCore;

namespace Escali.LicenseControl.Client.ViewModels
{
    public class ModuleViewModel : IViewModel
    {

        private readonly ModuleService _moduleService;
        private AlertService _alertService;

        public List<ModuleClientModel>? Modules { get; private set; }
        public ModuleViewModel(DataContext context, IMapper mapper, AlertService alertService)
        {
            _moduleService = new ModuleService(context);
            _mapper = mapper;
            _alertService = alertService;
        }

        public async Task GetModules()
        {
            var modules = await _moduleService.GetAllModules();
            UpdateModules(_mapper.Map<List<ModuleClientModel>>(modules));
        }

        public async Task<ModuleClientModel?> CreateModule(ModuleClientModel moduleView)
        {
            var mapped = _mapper.Map<Module>(moduleView);
            try
            {
                var res = await _moduleService.AddModule(mapped);
                _alertService.UpdateInfoMessage(new InfoMessage("Ny modul lagt til!", InfoMessageType.SUCCESS));

                var updatedModules = Modules ?? new List<ModuleClientModel>();
                updatedModules.Add(_mapper.Map<ModuleClientModel>(res));

                UpdateModules(updatedModules);
                return _mapper.Map<ModuleClientModel>(res);
            }
            catch (DbUpdateException e)
            {
                _alertService.UpdateInfoMessage(new InfoMessage("Det oppstod en feil i database.", InfoMessageType.ERROR));
                return null;
            }
            catch (InvalidOperationException e)
            {
                _alertService.UpdateInfoMessage(new InfoMessage(e.Message, InfoMessageType.ERROR));
                return null;
            }
        }

        public async Task<ModuleClientModel>? UpdateModule(ModuleClientModel moduleView)
        {
            var mapped = _mapper.Map<Module>(moduleView);
            try
            {
                var res = await _moduleService.UpdateModule(mapped);
                _alertService.UpdateInfoMessage(new InfoMessage("Modul ble oppdatert!", InfoMessageType.SUCCESS));

                var updatedModules = Modules!.Select(m => m.ModuleSeq != moduleView.ModuleSeq ? m : _mapper.Map<ModuleClientModel>(res)).ToList();
                UpdateModules(updatedModules);
                return _mapper.Map<ModuleClientModel>(res);
            }
            catch (DbUpdateException e)
            {
                Logger.Error(e.Message);
                Console.WriteLine("Mapped modulelevelcount: " + mapped.ModuleLevels.Count);
                Console.WriteLine("Mapped ModuleSeq: " + mapped.ModuleSeq);
                _alertService.UpdateInfoMessage(new InfoMessage("Det oppstod en feil i database."
                    + "Tried to add ModuleLevel with modulelevelname :  " + mapped.ModuleLevels.FindLast(u => u.ModuleSeq == mapped.ModuleSeq).ModuleLevelName
                    , InfoMessageType.ERROR));
                return null;
            }
            catch (InvalidOperationException e)
            {
                Logger.Error(e.Message);
                _alertService.UpdateInfoMessage(new InfoMessage(e.Message, InfoMessageType.ERROR));
                return null;
            }
        }

        private void UpdateModules(List<ModuleClientModel> modules)
        {
            Modules = modules;
            OnDataChanged();
        }

    }
}