using AutoMapper;
using Escali.LicenseControl.Client.State;
using Escali.LicenseControl.Client.Utils;
using Escali.LicenseControl.Client.ClientModels;
using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Models;
using Escali.LicenseControl.DataAccess.Services;
using Microsoft.EntityFrameworkCore;

namespace Escali.LicenseControl.Client.ViewModels
{
    public class PriceListViewModel : IViewModel
    {
        private readonly PriceListService _priceListService;
        private readonly AlertService _alertService;

        public List<PriceListClientModel>? PriceLists { get; private set; }

        public PriceListViewModel(DataContext context, IMapper mapper, AlertService alertService)
        {
            _priceListService = new PriceListService(context);
            _mapper = mapper;
            _alertService = alertService;
        }

        public async Task GetPriceLists()
        {
            var priceLists = await _priceListService.GetAllPriceLists();
            UpdatePriceLists(_mapper.Map<List<PriceListClientModel>>(priceLists));
        }

        public async Task<PriceListClientModel?> CreatePricelist(PriceListClientModel priceList, int priceListToCopy, decimal priceAdjustmentFactor)
        {
            var mapped = _mapper.Map<PriceList>(priceList);
            try
            {
                var res = await _priceListService.AddPriceListFromCopy(mapped, priceListToCopy, priceAdjustmentFactor);
                _alertService.UpdateInfoMessage(new InfoMessage("Ny prisliste lagt til!", InfoMessageType.SUCCESS));

                var updatedPricelists = PriceLists ?? new List<PriceListClientModel>();
                updatedPricelists.Add(_mapper.Map<PriceListClientModel>(res));

                UpdatePriceLists(updatedPricelists);
                return _mapper.Map<PriceListClientModel>(res);
            }
            catch (DbUpdateException e)
            {
                _alertService.UpdateInfoMessage(new InfoMessage("Det oppstod en feil i database.", InfoMessageType.ERROR));
                return null;
            }
            catch (InvalidOperationException e)
            {
                _alertService.UpdateInfoMessage(new InfoMessage(e.Message, InfoMessageType.ERROR));
                return null;
            }
        }

        public async Task<PriceListClientModel?> UpdatePricelist(PriceListClientModel priceList)
        {
            var mapped = _mapper.Map<PriceList>(priceList);
            try
            {
                var res = await _priceListService.UpdatePriceList(mapped);
                _alertService.UpdateInfoMessage(new InfoMessage("Prisliste ble oppdatert!", InfoMessageType.SUCCESS));

                var updatedPricelists = PriceLists;
                updatedPricelists = updatedPricelists!.Select(pl => pl.PriceListSeq != priceList.PriceListSeq ? pl : _mapper.Map<PriceListClientModel>(res)).ToList();

                UpdatePriceLists(updatedPricelists);
                return _mapper.Map<PriceListClientModel>(res);
            }
            catch (DbUpdateException e)
            {
                _alertService.UpdateInfoMessage(new InfoMessage("Det oppstod en feil i database.", InfoMessageType.ERROR));
                return null;
            }
            catch (InvalidOperationException e)
            {
                _alertService.UpdateInfoMessage(new InfoMessage(e.Message, InfoMessageType.ERROR));
                return null;
            }
        }

        private void UpdatePriceLists(List<PriceListClientModel> priceLists)
        {
            PriceLists = priceLists;
            OnDataChanged();
        }

    }
}