using System;
using Escali.LicenseControl.DataAccess.Models;
using Escali.LicenseControl.DataAccess.Services;
using Escali.LicenseControl.DataAccess.Tests.TestUtils;
using Xunit;

namespace Escali.LicenseControl.DataAccess.Tests.Services
{

/// <summary>
/// AgreementService Testing Environment
/// </summary>
public class AgreementServiceTest
{

    /// <summary>
    /// Creates an agreement
    /// </summary>
    /// <returns>Agreement</returns>
    private Agreement CreateAgreement()
    {
        return new Agreement()
        {
            AgreementName = "Escali Financials",
            AgreementCreated = DateTime.Now,
            CurrencySeq = 1,
            ProductSeq = 1,
            CustomerSeq = 1
        };
    }

    /// <summary>
    /// passes if agreement is correctly inserted into agreementlist
    /// </summary>
    [Fact]
    public async void AddAgreement_Success()
    {
        var context = TestBase.GetContext();
        TestData.InsertDataForAgreement(context);

        var agreement = CreateAgreement();

        var service = new AgreementService(context);
        var res = await service.AddAgreement(agreement);

        Assert.Equal(agreement.AgreementName, res.AgreementName);
        Assert.NotNull(res.Currency);
        Assert.NotNull(res.Product);
        Assert.NotNull(res.Customer);
    }
    
    /// <summary>
    /// passes if correctly added agreeement
    /// </summary>
    [Fact]
    public async void AddAgreement_DateChangedAdded()
    {
        var context = TestBase.GetContext();
        TestData.InsertDataForAgreement(context);

        var agreement = CreateAgreement();

        var service = new AgreementService(context);
        var res = await service.AddAgreement(agreement);

        Assert.NotNull(res.AgreementUpdated);
    }
    
    /// <summary>
    /// passes if correctly updated agreeement-date, and not equal previous set date
    /// </summary>
    [Fact]
    public async void UpdateAgreement_DateChangedAdded()
    {
        var context = TestBase.GetContext();
        TestData.InsertDataForAgreement(context);

        var agreement = CreateAgreement();
        agreement.AgreementUpdated = DateTime.Now;
        var res = context.Agreements.Add(agreement);
        await context.SaveChangesAsync();

        var lastChanged = res.Entity.AgreementUpdated;

        var service = new AgreementService(context);
        var updatedAgreement = await service.UpdateAgreement(res.Entity);

        Assert.NotEqual(lastChanged, updatedAgreement.AgreementUpdated);
    }

    }
}
