﻿using System.Threading.Tasks;
using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Models;
using Escali.LicenseControl.DataAccess.Services;
using Escali.LicenseControl.DataAccess.Tests.TestUtils;
using Xunit;

namespace Escali.LicenseControl.DataAccess.Tests.Services
{
    
/// <summary>
/// RegionService Testing Environment
/// </summary>
public class RegionServicetest
{
    /// <summary>
    /// Inserts two regions into the context
    /// </summary>
    /// <param name="context">DataContext</param>
    /// <returns>DataContext</returns>
    private async Task<DataContext> InsertData(DataContext context)
    {
        context.Regions.Add(new Region() { Country = "NOR", Area = "OSLO" });
        context.Regions.Add(new Region() { Country = "NOR", Area = "BERGEN" });
        await context.SaveChangesAsync();
        return context;
    }

    /// <summary>
    /// Passes if regionlist is not empty after inserting
    /// </summary>
    [Fact]
    public async void GetAllRegions_ContainingSameCountry()
    {
        // Arrange
        var context = TestBase.GetContext();
        await InsertData(context);

        // Act
        var regionService = new RegionService(context);
        var res = regionService.GetAllRegions();

        // Assert
        Assert.NotEmpty(res.Result);

    }

    /// <summary>
    /// Passes if retrieving regionlist is not empty and arenames of entries in regionlist is what is expected
    /// </summary>
    [Fact]
    public async void FindRegionById_CityEqualsOsloAndSecondEntryIsBergen()
    {
        var context = TestBase.GetContext();
        await InsertData(context);
        var reg = context.Regions.FindAsync(1).Result.Area;

        var regionService = new RegionService(context);
        var res = await regionService.GetAllRegions();

        Assert.NotEmpty(res);
        Assert.Equal(reg, res[0].Area);
        Assert.Equal("BERGEN", res[1].Area);
    }

    /// <summary>
    /// Passes if regionlist is notempty and region with id is found and the updated region has new expected name
    /// </summary>
    [Fact]
    public async void UpdateRegion_FindRegionByIdAndUpdateCity()
    {
        var context = TestBase.GetContext();
        await InsertData(context);

        var findIdOne = context.Regions.FindAsync(1).Result;

        var regionService = new RegionService(context);
        var res = await regionService.GetAllRegions();

        // Returns true on Bergen == Bergen
        Assert.Equal(res[0].Area, findIdOne.Area);

        findIdOne.Area = "Tønsberg";
        regionService.UpdateRegion(findIdOne);

        var resUpdate = await regionService.GetAllRegions();

        // Region Id 1 updated to Tønsberg, returns true on Tønsberg == Tønsberg
        Assert.Equal(resUpdate[0].Area, findIdOne.Area);
        // Confirms that the updated city with id=1 is no longer Bergen, return true on Bergen != Tønsberg
        Assert.NotEqual("BERGEN", resUpdate[0].Area);

    }

    }
}

