﻿using System;
using System.Threading.Tasks;
using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Models;
using Escali.LicenseControl.DataAccess.Services;
using Escali.LicenseControl.DataAccess.Tests.TestUtils;
using Xunit;

namespace Escali.LicenseControl.DataAccess.Tests.Services
{


/// <summary>
/// SegmentService Testing Environment
/// </summary>
public class SegmentServiceTest
{
    /// <summary>
    /// Adds an mainsegment with the segment to DataContext
    /// Will pass if user added correctly
    /// </summary>
    /// <param name="context">DataContext</param>
    /// <returns>DataContext</returns>
    private async Task<DataContext> InsertData(DataContext context)
    {
        context.MainSegments.Add(new MainSegment() { MainSegmentName = "MainSegment 1" });
        context.Segments.Add(new Segment () { SegmentName = "Segment 1", MainSegmentSeq = 1});
        await context.SaveChangesAsync();
        return context;
    }

    /// <summary>
    /// Adds an segment and mainsegment to DataContext
    /// Will pass if segmentlist is not empty
    /// </summary>
    [Fact]
    public async void AddSegment_SegmentListNotEmpty()
    {
        var context = TestBase.GetContext();
        await InsertData(context);

        var segmentService = new SegmentService(context);
        var res = segmentService.GetAllSegments();

        Assert.NotEmpty(res.Result);
    }

    /// <summary>
    /// Adds an segment and mainsegment to DataContext
    /// Will pass if InvalidOperationException is thrown
    /// </summary>
    [Fact]
    public async void AddSegment_SegmentNameAlreadyExists()
    {

        // Arrange
        var context = TestBase.GetContext();
        await InsertData(context);

        var newSegmentWithSameNameAsAlreadyInserted = new Segment() { SegmentName = "Segment 1"};
        await context.SaveChangesAsync();

        var segmentService = new SegmentService(context);

        // Act
        var op = async () => await segmentService.AddSegment(newSegmentWithSameNameAsAlreadyInserted);

        // Assert
        await Assert.ThrowsAsync<InvalidOperationException>(op);

    }

    /// <summary>
    /// Adds an segment and mainsegment to DataContext
    /// Will pass if updates segmentname equals the returned segmentname
    /// </summary>
    [Fact]
    public async void UpdateSegment_SegmentNameUpdatedCorrectly()
    {
        var context = TestBase.GetContext();
        await InsertData(context);

        var segmentService = new SegmentService(context);
        //var res = segmentService.GetAllSegments();

        var res = context.Segments.FindAsync(1).Result;

        res.SegmentName = "DetNyeSegmentet";
        segmentService.UpdateSegment(res);

        Assert.Equal(context.Segments.FindAsync(1).Result.SegmentName, res.SegmentName);


    }
    }
}

