﻿using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Models;
using Microsoft.EntityFrameworkCore;

namespace Escali.LicenseControl.DataAccess.Services
{

/// <summary>
/// PriorityService class inserts and updates Priority in the Database
/// </summary>
public class PriorityService
{

    private readonly DataContext _db;

    public PriorityService(DataContext db)
    {
        _db = db;
    }

    /// <summary>
    /// Read all Priorities from database
    /// </summary>
    /// <returns> A List of Priorities </returns>
    public async Task<List<Priority>> GetAllPriorities()
    {
        var res = await _db.Priorities.ToListAsync();
        return res;
    }

    /// <summary>
    /// Read one Priority with id from database
    /// </summary>
    /// <param name="id"></param>
    /// <returns> Priority with corresponding id </returns>
    /// <return> Null if id not found </return>
    public async Task<Priority> GetPriorityById(int id)
    {
        var res = await _db.Priorities.FindAsync(id);
        return res;
    }

    /// <summary>
    /// Adds a Priority to the database
    /// </summary>
    /// <param name="priority"></param>
    /// <returns> Priority Entity </returns>
    /// <exception cref="DbUpdateException"> Database: error encountered while saving to database </exception>
    /// <exception cref="DbUpdateConcurrencyException"> Database: not updated database before new update </exception>
    public async Task<Priority> AddPriority(Priority priority)
    {
        var res = _db.Priorities.Add(priority);
        await _db.SaveChangesAsync();
        return res.Entity;
    }

    /// <summary>
    /// Updates changes on a Priority in the database
    /// </summary>
    /// <param name="priority"></param>
    /// <returns> The updated Priority entity </returns>
    /// <exception cref="DbUpdateException"> Database: error encountered while saving to database </exception>
    /// <exception cref="DbUpdateConcurrencyException"> Database: not updated database before new update </exception>
    public async Task<Priority> UpdatePriority(Priority priority)
    {
        var res = _db.Priorities.Update(priority);
        await _db.SaveChangesAsync();
        return res.Entity;
    }

    }
}
