﻿using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Models;
using Microsoft.EntityFrameworkCore;

namespace Escali.LicenseControl.DataAccess.Services
{

/// <summary>
/// RegionService class inserts and updates Region in the Database
/// </summary>
public class RegionService
{

    public readonly DataContext _db;

    public RegionService(DataContext db)
    {
        _db = db;
    }

    /// <summary>
    /// Read all Regions from database
    /// </summary>
    /// <returns> A List of Regions </returns>
    public async Task<List<Region>> GetAllRegions()
    {
        var res = await _db.Regions.ToListAsync();
        return res;
    }

    /// <summary>
    /// Read one Region with id from database
    /// </summary>
    /// <param name="id"></param>
    /// <returns> Region with corresponding id </returns>
    /// <return> Null if id not found </return>
    public async Task<Region> GetRegionById(int id)
    {
        var res = await _db.Regions.FindAsync(id);
        return res;
    }

    /// <summary>
    /// Adds a Region to the database
    /// </summary>
    /// <param name="region"></param>
    /// <returns> Region Entity </returns>
    /// <exception cref="DbUpdateException"> Database: error encountered while saving to database </exception>
    /// <exception cref="DbUpdateConcurrencyException"> Database: not updated database before new update </exception>
    public async Task<Region> AddRegion(Region region)
    {
        if (_db.Regions.Any(r => r.Area == region.Area && r.Country == region.Country))
            throw new InvalidOperationException($"Region name already exist");

        var res = _db.Regions.Add(region);  
        await _db.SaveChangesAsync();
        return res.Entity;
    }

    /// <summary>
    /// Updates changes on a Region in the database
    /// </summary>
    /// <param name="region"></param>
    /// <returns> The updated Region entity </returns>
    /// <exception cref="DbUpdateException"> Database: error encountered while saving to database </exception>
    /// <exception cref="DbUpdateConcurrencyException"> Database: not updated database before new update </exception>
    public async Task<Region> UpdateRegion(Region region)
    {
        if (_db.Regions.Any(r => r.Area == region.Area && r.Country != region.Country))
            throw new InvalidOperationException($"update region error");

        var res = _db.Regions.Update(region);
        await _db.SaveChangesAsync();
        return res.Entity;
    }
    


}

}
