﻿using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Models;
using Microsoft.EntityFrameworkCore;

namespace Escali.LicenseControl.DataAccess.Services
{

/// <summary>
/// SegmentService class inserts and updates Segment in the Database
/// </summary>
public class SegmentService
{

    private readonly DataContext _db;

    public SegmentService(DataContext db)
    {
        _db = db;
    }

    /// <summary>
    /// Read all Segments from database
    /// </summary>
    /// <returns> A List of Segments </returns>
    public async Task<List<Segment>> GetAllSegments()
    {
        var res = await _db.Segments.ToListAsync();
        return res;
    }

    /// <summary>
    /// Read one Segment with id from database
    /// </summary>
    /// <param name="id"></param>
    /// <returns> Segment with corresponding id </returns>
    /// <return> Null if id not found </return>
    public async Task<Segment> GetSegmentById(int id)
    {
        var res = await _db.Segments.FindAsync(id);
        return res;
    }

    /// <summary>
    /// Adds a Segment to the database
    /// </summary>
    /// <param name="segment"></param>
    /// <returns> Segment Entity </returns>
    /// <exception cref="DbUpdateException"> Database: error encountered while saving to database </exception>
    /// <exception cref="DbUpdateConcurrencyException"> Database: not updated database before new update </exception>
    public async Task<Segment> AddSegment(Segment segment)
    {
        if (_db.Segments.Any(s => s.SegmentName == segment.SegmentName))
            throw new InvalidOperationException($"Segmnet name already exist");

        var res = _db.Segments.Add(segment);
        await _db.SaveChangesAsync();
        return res.Entity;
    }

    /// <summary>
    /// Updates changes on a Segment in the database
    /// </summary>
    /// <param name="segment"></param>
    /// <returns> The updated Segment entity </returns>
    /// <exception cref="DbUpdateException"> Database: error encountered while saving to database </exception>
    /// <exception cref="DbUpdateConcurrencyException"> Database: not updated database before new update </exception>
    public async Task<Segment> UpdateSegment(Segment segment)
    {
        if (_db.Segments.Any(s => s.SegmentName == segment.SegmentName && s.MainSegmentSeq == segment.MainSegmentSeq))
            throw new InvalidOperationException("Could not update segment");

        var res = _db.Segments.Update(segment);
        await _db.SaveChangesAsync();
        return res.Entity;
    }

}

}
