﻿using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Models;
using Microsoft.EntityFrameworkCore;

namespace Escali.LicenseControl.DataAccess.Services
{

/// <summary>
/// UserService class inserts and updates Users in the Database
/// </summary>
public class UserService
{
    private readonly DataContext _db;

    public UserService(DataContext db)
    {
        _db = db;
    }

    /// <summary>
    /// Read all Users from database
    /// </summary>
    /// <returns> List of Users </returns>
    public async Task<List<User>> GetAllUsers()
    {
        var res = await _db.Users.ToListAsync();
        return res;
    }

    /// <summary>
    /// Read one User with id from database
    /// </summary>
    /// <param name="id"></param>
    /// <returns> User with corresponding id </returns>
    /// <return> Null if id not found </return>
    public async Task<User> GetUserById(int id)
    {
        var res = await _db.Users.FindAsync(id);
        return res;
    }

    /// <summary>
    /// Adds a User to the database
    /// </summary>
    /// <param name="user"></param>
    /// <returns> User Entity </returns>
    /// <exception cref="DbUpdateException"> Database: error encountered while saving to database </exception>
    /// <exception cref="DbUpdateConcurrencyException"> Database: not updated database before new update </exception>
    public async Task<User> AddUser(User user)
    {
        if (_db.Users.Any(u => u.UserEmail == user.UserEmail))
            throw new InvalidOperationException($"Email already exist");

        var res = _db.Users.Add(user);
        await _db.SaveChangesAsync();
        return res.Entity;
    }

    /// <summary>
    /// Updates changes on a User in the database
    /// </summary>
    /// <param name="user"></param>
    /// <returns> The updated User entity </returns>
    /// <exception cref="DbUpdateException"> Database: error encountered while saving to database </exception>
    /// <exception cref="DbUpdateConcurrencyException"> Database: not updated database before new update </exception>
    public async Task<User> UpdateUser(User user)
    {
        if (_db.Users.Any(u => u.UserEmail == user.UserEmail))
            throw new InvalidOperationException($"Email already exist");

        var res = _db.Users.Update(user);
        await _db.SaveChangesAsync();
        return res.Entity;
    }

    }

}