﻿using Escali.LicenseControl.DataAccess.Data;
using Escali.LicenseControl.DataAccess.Models;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.DependencyInjection;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Escali.LicenseControl.DataAccess.Utils
{

public class DataGenerator
{

    public static void Initialize(IServiceProvider serviceProvider)
    {
        using (var context = new DataContext(
            serviceProvider.GetRequiredService<DbContextOptions<DataContext>>()))
        {
            if (context.Modules.Any())
            {
                return;
            }

            // Users
            context.Users.AddRange(
                new User
                {
                    UserEmail = "rune@gmail.com",
                    UserConsulentName = "Rune"
                },
                new User
                {
                    UserEmail = "simen@gmail.com",
                    UserConsulentName = "Simen"
                },
                new User
                {
                    UserEmail = "gerdt@gmail.com",
                    UserConsulentName = "Gerdt"
                }
            );

            // Products
            context.Products.AddRange(
                new Product
                {
                    ProductName = "Escali Supervision"
                },
                new Product
                {
                    ProductName = "Escali Financials"
                }
            );

            // Currencies
            context.Currencies.AddRange(
                new Currency
                {
                    CurrencyName = "NOK"
                },
                new Currency
                {
                    CurrencyName = "SEK"
                },
                new Currency
                {
                    CurrencyName = "EUR"
                }
            );

            // Priorities
            context.Priorities.AddRange(
                new Priority
                {
                    PriorityName = Priority.PriorityLevel.SILVER
                },
                new Priority
                {
                    PriorityName = Priority.PriorityLevel.GOLD
                },
                new Priority
                {
                    PriorityName = Priority.PriorityLevel.PLATINUM
                }
            );

                        // Mainsegments
            context.MainSegments.AddRange(
                new MainSegment
                {
                    MainSegmentName = "Store virksomheter"
                },
                 new MainSegment
                 {
                     MainSegmentName = "Bank og finans"
                 },
                  new MainSegment
                  {
                      MainSegmentName = "Offentlig forvaltning"
                  }

            );

            // Segments
            context.Segments.AddRange(
                new Segment
                {
                    SegmentName = "Pensjonskasser",
                    MainSegmentSeq = 2
                },
                new Segment
                {
                    SegmentName = "Skadeforsikringsselskaper",
                    MainSegmentSeq = 2
                },
                new Segment
                {
                    SegmentName = "Banker",
                    MainSegmentSeq = 2
                },
                new Segment
                {
                    SegmentName = "Shippingselskaper",
                    MainSegmentSeq = 1
                },
                new Segment
                {
                    SegmentName = "Eieidomsselskaper",
                    MainSegmentSeq = 1
                },
                new Segment
                {
                    SegmentName = "Offshoreselskaper",
                    MainSegmentSeq = 1
                },
                new Segment
                {
                    SegmentName = "Kommuner",
                    MainSegmentSeq = 3
                },
                new Segment
                {
                    SegmentName = "Fylkeskommuner",
                    MainSegmentSeq = 3
                },
                new Segment
                {
                    SegmentName = "Annen offentlig forvaltning",
                    MainSegmentSeq = 3
                }
            );

            // Regions
            context.Regions.AddRange(
                new Region
                {
                    Country = "Norge",
                    Area = "Oslo og omegn"
                },
                new Region
                {
                    Country = "Norge",
                    Area = "Bergen og omegn"
                },
                new Region
                {
                    Country = "Norge",
                    Area = "Nordvest og omegn"
                },
                new Region
                {
                    Country = "Norge",
                    Area = "Indre østland"
                },
                new Region
                {
                    Country = "Sverige",
                    Area = "Stockholm"
                },
                new Region
                {
                    Country = "Norge",
                    Area = "Midtnorge"
                }
            );

            // Customers
            context.Customers.AddRange(
                new Customer
                {
                    OrganizationNumber = "10278",
                    CustomerName = "Ferde AS",
                    LedgerNumber = 22222,
                    Connection = "ferde:connection",
                    DBName = "ferde:db",
                    CustomerUsername = "userFerde",
                    CustomerPassword = "passFerde",
                    API = "ferdeAPI",
                    CustomerSince = DateTime.Now,
                    IsActive = true,
                    PrioritySeq = 3,
                    SegmentSeq = 9,
                    RegionSeq = 2,
                    UserSeq = 3
                }
            );

            // Modules
            context.Modules.AddRange(
                new Module
                {
                    ModuleName = "Shares and funds",
                    IsActive = true,
                    ProductSeq = 2,
                },
                new Module
                {
                    ModuleName = "Bonds",
                    IsActive = true,
                    ProductSeq = 2,
                },
                new Module
                {
                    ModuleName = "Depts",
                    IsActive = false,
                    ProductSeq = 2,
                },
                new Module
                {
                    ModuleName = "Supervision",
                    IsActive = true,
                    ProductSeq = 1,
                },
                new Module
                {
                    ModuleName = "Supervision - General",
                    IsActive = false,
                    ProductSeq = 1,
                }
            );

            // ModuleLevels
            context.ModuleLevels.AddRange(
                new ModuleLevel
                {
                    ModuleLevelName = "EF Aksje-/fondsmodul 1",
                    ModuleLevelRestriction = "MAX 250M/100tr",
                    ModuleSeq = 1,
                    IsActive = true
                },
                new ModuleLevel
                {
                    ModuleLevelName = "EF Aksje-/fondsmodul 2",
                    ModuleLevelRestriction = "MAX 250M/250tr",
                    ModuleSeq = 1,
                    IsActive = true
                },
                new ModuleLevel
                {
                    ModuleLevelName = "EF Aksje-/fondsmodul 3",
                    ModuleLevelRestriction = "MAX 500M/500tr",
                    ModuleSeq = 1,
                    IsActive = true
                },
                new ModuleLevel
                {
                    ModuleLevelName = "EF Aksje-/fondsmodul 4",
                    ModuleLevelRestriction = "MAX 1MRD/1000tr",
                    ModuleSeq = 1,
                    IsActive = true
                },
                new ModuleLevel
                {
                    ModuleLevelName = "EF Bonds 1",
                    ModuleLevelRestriction = "MAX 250M/100tr",
                    ModuleSeq = 2,
                    IsActive = true
                },
                new ModuleLevel
                {
                    ModuleLevelName = "EF Bonds 2",
                    ModuleLevelRestriction = "MAX 250M/250tr",
                    ModuleSeq = 2,
                    IsActive = true
                },
                new ModuleLevel
                {
                    ModuleLevelName = "EF Bonds 3",
                    ModuleLevelRestriction = "MAX 1MRD/500tr",
                    ModuleSeq = 2,
                    IsActive = true
                },
                new ModuleLevel
                {
                    ModuleLevelName = "EF Bonds 4",
                    ModuleLevelRestriction = "MAX 2.5MRD/1000tr",
                    ModuleSeq = 2,
                    IsActive = true
                },
                new ModuleLevel
                {
                    ModuleLevelName = "EF Debt 1",
                    ModuleLevelRestriction = "MAX 10 loans",
                    ModuleSeq = 3,
                    IsActive = true
                },
                new ModuleLevel
                {
                    ModuleLevelName = "EF Debt 2",
                    ModuleLevelRestriction = "MAX 25 loans",
                    ModuleSeq = 3,
                    IsActive = true
                },
                new ModuleLevel
                {
                    ModuleLevelName = "EF Debt 3",
                    ModuleLevelRestriction = "MAX 50 loans",
                    ModuleSeq = 3,
                    IsActive = true
                },
                new ModuleLevel
                {
                    ModuleLevelName = "EF Debt 4",
                    ModuleLevelRestriction = "MAX 100 loans",
                    ModuleSeq = 3,
                    IsActive = true
                },
                new ModuleLevel
                {
                    ModuleLevelName = "Escali Supervision",
                    ModuleLevelRestriction = "EIOPA",
                    ModuleSeq = 4,
                    IsActive = true
                },
                new ModuleLevel
                {
                    ModuleLevelName = "Escali Supervision2",
                    ModuleLevelRestriction = "Integrasjonsmodul eksternt porteføljesystem",
                    ModuleSeq = 4,
                    IsActive = true
                },
                new ModuleLevel
                {
                    ModuleLevelName = "Supervision Finans Modul",
                    ModuleLevelRestriction = "500-1000 posisjoner",
                    ModuleSeq = 5,
                    IsActive = true
                },
                new ModuleLevel
                {
                    ModuleLevelName = "Supervision Fondsinnholdmodul",
                    ModuleLevelRestriction = "None",
                    ModuleSeq = 5,
                    IsActive = true
                },
                new ModuleLevel
                {
                    ModuleLevelName = "Supervision Fort",
                    ModuleLevelRestriction = "None",
                    ModuleSeq = 5,
                    IsActive = true
                }
            );

            // PriceLists
            context.PriceLists.AddRange(
                    new PriceList
                    {
                        PriceListName = "Prisliste 2021",
                        DateFrom = DateTime.Parse("2021-01-01 12:00:00"),
                        DateTo = DateTime.Parse("2021-12-31 12:00:00"),
                        IsActive = false,
                        CurrencySeq = 1,
                        ProductSeq = 2
                    },
                    new PriceList
                    {
                        PriceListName = "Prisliste 2022",
                        DateFrom = DateTime.Parse("2022-01-01 12:00:00"),
                        DateTo = DateTime.Parse("2022-12-31 12:00:00"),
                        IsActive = true,
                        CurrencySeq = 1,
                        ProductSeq = 2
                    },
                    new PriceList
                    {
                        PriceListName = "Prisliste Super 2021",
                        DateFrom = DateTime.Parse("2021-01-01 12:00:00"),
                        DateTo = DateTime.Parse("2021-12-31 12:00:00"),
                        IsActive = false,
                        CurrencySeq = 1,
                        ProductSeq = 1
                    },
                    new PriceList
                    {
                        PriceListName = "Prisliste Super 2022",
                        DateFrom = DateTime.Parse("2022-01-01 12:00:00"),
                        DateTo = DateTime.Parse("2022-12-31 12:00:00"),
                        IsActive = true,
                        CurrencySeq = 1,
                        ProductSeq = 1
                    }
            );

            // PriceElements
            context.PriceElements.AddRange(
                // 1
                new PriceElement
                {
                    Price = 450,
                    IsActive = true,
                    ModuleLevelSeq = 1,
                    PriceListSeq = 1
                },
                new PriceElement
                {
                    Price = 750,
                    IsActive = true,
                    ModuleLevelSeq = 2,
                    PriceListSeq = 1
                },
                new PriceElement
                {
                    Price = 900,
                    IsActive = true,
                    ModuleLevelSeq = 3,
                    PriceListSeq = 1
                },
                new PriceElement
                {
                    Price = 1250,
                    IsActive = true,
                    ModuleLevelSeq = 4,
                    PriceListSeq = 1
                },

                // 2
                new PriceElement
                {
                    Price = 500,
                    IsActive = true,
                    ModuleLevelSeq = 1,
                    PriceListSeq = 2
                },
                new PriceElement
                {
                    Price = 800,
                    IsActive = true,
                    ModuleLevelSeq = 2,
                    PriceListSeq = 2
                },
                new PriceElement
                {
                    Price = 1000,
                    IsActive = true,
                    ModuleLevelSeq = 3,
                    PriceListSeq = 2
                },
                new PriceElement
                {
                    Price = 1300,
                    IsActive = true,
                    ModuleLevelSeq = 4,
                    PriceListSeq = 2
                },

                // 3
                new PriceElement
                {
                    Price = 450,
                    IsActive = true,
                    ModuleLevelSeq = 5,
                    PriceListSeq = 1
                },
                new PriceElement
                {
                    Price = 750,
                    IsActive = true,
                    ModuleLevelSeq = 6,
                    PriceListSeq = 1
                },
                new PriceElement
                {
                    Price = 900,
                    IsActive = true,
                    ModuleLevelSeq = 7,
                    PriceListSeq = 1
                },
                new PriceElement
                {
                    Price = 1250,
                    IsActive = true,
                    ModuleLevelSeq = 8,
                    PriceListSeq = 1
                },

                // 4
                new PriceElement
                {
                    Price = 500,
                    IsActive = true,
                    ModuleLevelSeq = 5,
                    PriceListSeq = 2
                },
                new PriceElement
                {
                    Price = 800,
                    IsActive = true,
                    ModuleLevelSeq = 6,
                    PriceListSeq = 2
                },
                new PriceElement
                {
                    Price = 1000,
                    IsActive = true,
                    ModuleLevelSeq = 7,
                    PriceListSeq = 2
                },
                new PriceElement
                {
                    Price = 1300,
                    IsActive = true,
                    ModuleLevelSeq = 8,
                    PriceListSeq = 2
                },

                // 5
                new PriceElement
                {
                    Price = 450,
                    IsActive = true,
                    ModuleLevelSeq = 9,
                    PriceListSeq = 1
                },
                new PriceElement
                {
                    Price = 750,
                    IsActive = true,
                    ModuleLevelSeq = 10,
                    PriceListSeq = 1
                },
                new PriceElement
                {
                    Price = 900,
                    IsActive = true,
                    ModuleLevelSeq = 11,
                    PriceListSeq = 1
                },
                new PriceElement
                {
                    Price = 1250,
                    IsActive = true,
                    ModuleLevelSeq = 12,
                    PriceListSeq = 1
                },

                // 6
                new PriceElement
                {
                    Price = 500,
                    IsActive = true,
                    ModuleLevelSeq = 9,
                    PriceListSeq = 2
                },
                new PriceElement
                {
                    Price = 800,
                    IsActive = true,
                    ModuleLevelSeq = 10,
                    PriceListSeq = 2
                },
                new PriceElement
                {
                    Price = 1000,
                    IsActive = true,
                    ModuleLevelSeq = 11,
                    PriceListSeq = 2
                },
                new PriceElement
                {
                    Price = 1300,
                    IsActive = true,
                    ModuleLevelSeq = 12,
                    PriceListSeq = 2
                },

                // 7
                new PriceElement
                {
                    Price = 500,
                    IsActive = true,
                    ModuleLevelSeq = 13,
                    PriceListSeq = 3
                },
                new PriceElement
                {
                    Price = 800,
                    IsActive = true,
                    ModuleLevelSeq = 14,
                    PriceListSeq = 3
                },
                new PriceElement
                {
                    Price = 500,
                    IsActive = true,
                    ModuleLevelSeq = 15,
                    PriceListSeq = 3
                },
                new PriceElement
                {
                    Price = 800,
                    IsActive = true,
                    ModuleLevelSeq = 16,
                    PriceListSeq = 3
                }, 
                new PriceElement
                {
                    Price = 1300,
                    IsActive = true,
                    ModuleLevelSeq = 17,
                    PriceListSeq = 3
                },

                // 8
                new PriceElement
                {
                    Price = 550,
                    IsActive = true,
                    ModuleLevelSeq = 13,
                    PriceListSeq = 4
                },
                new PriceElement
                {
                    Price = 900,
                    IsActive = true,
                    ModuleLevelSeq = 14,
                    PriceListSeq = 4
                },
                new PriceElement
                {
                    Price = 600,
                    IsActive = true,
                    ModuleLevelSeq = 15,
                    PriceListSeq = 4
                },
                new PriceElement
                {
                    Price = 850,
                    IsActive = true,
                    ModuleLevelSeq = 16,
                    PriceListSeq = 4
                },
                new PriceElement
                {
                    Price = 1500,
                    IsActive = true,
                    ModuleLevelSeq = 17,
                    PriceListSeq = 4
                }
            );
            context.SaveChanges();
        }
    }

    }
}

